@@ -66,22 +66,24 @@
 
 //	-----------------------------------------------------------------
 #ifdef DEBUGLOG
+#define SLEEPTIME		20
+//#define SLEEPTIME		0
 #define debugIOLog( message ) \
-	{IOLog( message ); IOSleep(20);}
+	{IOLog( message ); IOSleep(SLEEPTIME);}
 #define debug2IOLog( message, arg2 ) \
-	{IOLog( message, arg2 ); IOSleep(20);}
+	{IOLog( message, arg2 ); IOSleep(SLEEPTIME);}
 #define debug3IOLog( message, arg2, arg3 ) \
-	{IOLog( message, arg2, arg3 ); IOSleep(20);}
+	{IOLog( message, arg2, arg3 ); IOSleep(SLEEPTIME);}
 #define debug4IOLog( message, arg2, arg3, arg4 ) \
-	{IOLog( message, arg2, arg3, arg4 ); IOSleep(20);}
+	{IOLog( message, arg2, arg3, arg4 ); IOSleep(SLEEPTIME);}
 #define debug5IOLog( message, arg2, arg3, arg4, arg5 ) \
-	{IOLog( message, arg2, arg3, arg4, arg5 ); IOSleep(20);}
+	{IOLog( message, arg2, arg3, arg4, arg5 ); IOSleep(SLEEPTIME);}
 #define debug6IOLog( message, arg2, arg3, arg4, arg5, arg6 ) \
-	{IOLog( message, arg2, arg3, arg4, arg5, arg6 ); IOSleep(20);}
+	{IOLog( message, arg2, arg3, arg4, arg5, arg6 ); IOSleep(SLEEPTIME);}
 #define debug7IOLog( message, arg2, arg3, arg4, arg5, arg6, arg7 ) \
-	{IOLog( message, arg2, arg3, arg4, arg5, arg6, arg7 ); IOSleep(20);}
+	{IOLog( message, arg2, arg3, arg4, arg5, arg6, arg7 ); IOSleep(SLEEPTIME);}
 #define debug8IOLog( message, arg2, arg3, arg4, arg5, arg6, arg7, arg8 ) \
-	{IOLog( message, arg2, arg3, arg4, arg5, arg6, arg7, arg8 ); IOSleep(20);}
+	{IOLog( message, arg2, arg3, arg4, arg5, arg6, arg7, arg8 ); IOSleep(SLEEPTIME);}
 #else
 #define debugIOLog( message ) ;
 #define debug2IOLog( message, arg2 ) ;
