@@ -15,24 +15,7 @@
 //
 //--------------------------------------------------------------------------------
 
-#include "AppleUSBAudioCommon.h"
 #include "AppleUSBAudioDevice.h"
-#include "AppleUSBAudioLevelControl.h"
-#include "AppleUSBAudioMuteControl.h"
-#include "USBAudioObject.h"
-#include "AppleUSBAudioEngine.h"
-
-#include <IOKit/IOLib.h>
-#include <IOKit/IOKitKeys.h>
-#include <IOKit/IORegistryEntry.h>
-#include <IOKit/audio/IOAudioStream.h>
-#include <IOKit/audio/IOAudioDefines.h>
-
-#include <IOKit/usb/IOUSBInterface.h>
-#include <IOKit/usb/IOUSBRootHubDevice.h>
-#include <IOKit/IOMessage.h>
-
-#include <libkern/c++/OSCollectionIterator.h>
 
 #define super IOAudioDevice
 
@@ -57,48 +40,14 @@ Exit:
 void AppleUSBAudioDevice::free () {
     debug2IOLog ("+AppleUSBAudioDevice[%p]::free ()\n", this);
 
-#if DEBUG
-	if (interfaceVendor && interfaceVendor->getRetainCount () == 0) Debugger ("interfaceVendor = 0");
-	if (interfaceProduct && interfaceProduct->getRetainCount () == 0) Debugger ("interfaceProduct = 0");
-	if (deviceReleaseNumber && deviceReleaseNumber->getRetainCount () == 0) Debugger ("deviceReleaseNumber = 0");
-	if (configurationValue && configurationValue->getRetainCount () == 0) Debugger ("configurationValue = 0");
-	if (interfaceNumber && interfaceNumber->getRetainCount () == 0) Debugger ("interfaceNumber = 0");
-	if (usbAudio && usbAudio->getRetainCount () == 0) Debugger ("usbAudio = 0");
-#endif
-
     if (interfaceLock) {
         IORecursiveLockFree (interfaceLock);
         interfaceLock = NULL;
     }
 
-    if (interfaceVendor) {
-        interfaceVendor->release ();
-        interfaceVendor = 0;
-    }
-
-    if (interfaceProduct) {
-        interfaceProduct->release ();
-        interfaceProduct = 0;
-    }
-
-    if (deviceReleaseNumber) {
-        deviceReleaseNumber->release ();
-        deviceReleaseNumber = 0;
-    }
-
-    if (configurationValue) {
-        configurationValue->release ();
-        configurationValue = 0;
-    }
-
-    if (interfaceNumber) {
-        interfaceNumber->release ();
-        interfaceNumber = 0;
-    }
-
     if (usbAudio) {
         usbAudio->release ();
-        usbAudio = 0;
+        usbAudio = NULL;
     }
 
     super::free ();
@@ -111,35 +60,131 @@ bool AppleUSBAudioDevice::requestTerminate (IOService * provider, IOOptionBits o
 	return TRUE;		// it is OK to terminate us
 }
 
+bool AppleUSBAudioDevice::ControlsStreamNumber (UInt8 streamNumber) {
+	UInt8 *							streamNumbers;
+	UInt8							numStreams;
+	UInt8							index;
+	bool							doesControl;
+
+	doesControl = FALSE;
+
+	if (usbAudio) {
+		usbAudio->GetControlledStreamNumbers (&streamNumbers, &numStreams);
+		for (index = 0; index < numStreams; index++) {
+			debug3IOLog ("Checking stream %d against controled stream %d\n", streamNumber, streamNumbers[index]);
+			if (streamNumber == streamNumbers[index]) {
+				doesControl = TRUE;
+				break;				// get out of for loop
+			}
+		}
+	}
+
+	return doesControl;
+}
+
 bool AppleUSBAudioDevice::start (IOService * provider) {
-    OSObject *						obj;
-	UInt8							stringIndex;
+	bool								result;
+
+    debug3IOLog ("+ AppleUSBAudioDevice[%p]::start (%p)\n", this, provider);
+	result = FALSE;
+
+	controlInterface = OSDynamicCast (IOUSBInterface, provider);
+	FailIf (FALSE == controlInterface->open (this), Exit);
+
+	mInitHardwareThread = thread_call_allocate ((thread_call_func_t)AppleUSBAudioDevice::initHardwareThread, (thread_call_param_t)this);
+	FailIf (NULL == mInitHardwareThread, Exit);
+
+    debug3IOLog ("- AppleUSBAudioDevice[%p]::start (%p)\n", this, provider);
+
+	result = super::start (provider);				// Causes our initHardware routine to be called.
+
+Exit:
+	return result;
+}
+
+bool AppleUSBAudioDevice::initHardware (IOService * provider) {
+	bool								result;
+
+	result = FALSE;
+
+	FailIf (NULL == mInitHardwareThread, Exit);
+	thread_call_enter1 (mInitHardwareThread, (void *)provider);
+
+	result = TRUE;
+
+Exit:
+	return result;
+}
+
+void AppleUSBAudioDevice::initHardwareThread (AppleUSBAudioDevice * aua, void * provider) {
+	IOCommandGate *						cg;
+	IOReturn							result;
+
+	FailIf (NULL == aua, Exit);
+//	FailIf (TRUE == aua->mTerminating, Exit);	
+
+	cg = aua->getCommandGate ();
+	if (cg) {
+		result = cg->runAction (aua->initHardwareThreadAction, provider);
+	}
+
+Exit:
+	return;
+}
+
+IOReturn AppleUSBAudioDevice::initHardwareThreadAction (OSObject * owner, void * provider, void * arg2, void * arg3, void * arg4) {
+	AppleUSBAudioDevice *				aua;
+	IOReturn							result;
+
+	result = kIOReturnError;
+
+	aua = (AppleUSBAudioDevice *)owner;
+	FailIf (NULL == aua, Exit);
+
+	result = aua->protectedInitHardware ((IOService *)provider);
+
+Exit:
+	return result;
+}
+
+IOReturn AppleUSBAudioDevice::protectedInitHardware (IOService * provider) {
 	char							string[kStringBufferSize];
+	UInt8							stringIndex;
 	IOReturn						err;
     Boolean							resultCode;
+	UInt8 *							streamNumbers;
+	UInt8							numStreams;
 
 	debug3IOLog ("+AppleUSBAudioDevice[%p]::start (%p)\n", this, provider);
 
 	resultCode = FALSE;
 
-	controlInterface = OSDynamicCast (IOUSBInterface, provider);
-	FailIf (NULL == controlInterface, Exit);
-	previousInterface = controlInterface;
-
 	debug2IOLog ("There are %d configurations on this device\n", controlInterface->GetDevice()->GetNumConfigurations ());
-	usbAudio = USBAudioConfigObject::create (controlInterface->GetDevice()->GetFullConfigurationDescriptor (0));
+	debug2IOLog ("Our control interface number is %d\n", controlInterface->GetInterfaceNumber ());
+	usbAudio = USBAudioConfigObject::create (controlInterface->GetDevice()->GetFullConfigurationDescriptor (0), controlInterface->GetInterfaceNumber ());
 	FailIf (NULL == usbAudio, Exit);
 
+	// Check to make sure that the control interface we loaded against has audio streaming interfaces and not just MIDI.
+	usbAudio->GetControlledStreamNumbers (&streamNumbers, &numStreams);
+	debug2IOLog ("Num streams controlled = %d\n", numStreams);
+	debug2IOLog ("GetNumStreamInterfaces = %d\n", usbAudio->GetNumStreamInterfaces ());
+	FailIf (0 == numStreams, Exit);
+
 	// If this is an iSub, we need to not go any further because we don't support it in this driver
 	// This will cause the driver to not load on any device that has _only_ a low frequency effect output terminal
 	FailIf (usbAudio->GetNumOutputTerminals (0, 0) == 1 && usbAudio->GetIndexedOutputTerminalType (0, 0, 0) == OUTPUT_LOW_FREQUENCY_EFFECTS_SPEAKER, Exit);
 
 	err = kIOReturnError;
 	string[0] = 0;
+	stringIndex = controlInterface->GetInterfaceStringIndex ();
+	if (0 != stringIndex) {
+		err = controlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
+	} else {
 	stringIndex = controlInterface->GetDevice()->GetProductStringIndex ();
 	if (0 != stringIndex) {
 		err = controlInterface->GetDevice()->GetStringDescriptor (stringIndex, string, kStringBufferSize);
 	}
+	}
 
 	if (0 == string[0] || kIOReturnSuccess != err) {
 		strcpy (string, "Unknown USB Audio Device");
@@ -161,45 +206,17 @@ bool AppleUSBAudioDevice::start (IOService * provider) {
 	setManufacturerName (string);
 	setDeviceTransportType (kIOAudioDeviceTransportTypeUSB);
 
-	FailIf (FALSE == controlInterface->open (this), Exit);
-	// get the IOAudioDevice generic start routine
-	FailIf (FALSE == super::start (provider), Exit);
-
 	interfaceLock = IORecursiveLockAlloc ();
 	FailIf (NULL == interfaceLock, Exit);
 
-	if (obj = controlInterface->getProperty (kUSBVendorName)) {
-		obj->retain ();
-		interfaceVendor = obj;
-	}
-
-	if (obj = controlInterface->getProperty (kUSBProductName)) {
-		obj->retain ();
-		interfaceProduct = obj;
-	}
-
-	if (obj = controlInterface->getProperty (kUSBDeviceReleaseNumber)) {
-		obj->retain ();
-		deviceReleaseNumber = obj;
-	}
-
-	if (obj = controlInterface->getProperty (kUSBConfigurationValue)) {
-		obj->retain();
-		configurationValue = obj;
-	}
-
-	if (obj = controlInterface->getProperty (kUSBInterfaceNumber)) {
-		obj->retain ();
-		interfaceNumber = obj;
-	}
-
-	resultCode = TRUE;
+	resultCode = super::initHardware (provider);
 
 Exit:
 	debug3IOLog ("-AppleUSBAudioDevice[%p]::start (%p)\n", this, provider);
-	return resultCode;
-}
 
+	return TRUE;
+}
+/*
 void AppleUSBAudioDevice::streamStopped (IOAudioEngine *audioEngine) {
     // This must be called from within the workLoop
     // The AppleUSBAudioEngine does so from _setState()
@@ -243,7 +260,7 @@ Exit:
     debug2IOLog ("-AppleUSBAudioDevice[%p]::allStreamsStopped ()\n", this);
     return result;
 }
-
+*/
 IOReturn AppleUSBAudioDevice::performPowerStateChange (IOAudioDevicePowerState oldPowerState, IOAudioDevicePowerState newPowerState, UInt32 *microSecsUntilComplete) {
 	IOReturn						result;
 
@@ -471,6 +488,8 @@ IOReturn AppleUSBAudioDevice::createControlsForInterface (IOAudioEngine *audioEn
 	UInt8								controlInterfaceNum;
 	Boolean								shouldUpdatePRAM;
 
+    debug4IOLog ("+AppleUSBAudioDevice[%p]::createControlsForInterface %d %d\n", this, interfaceNum, altInterfaceNum);
+
     result = kIOReturnError;
 	terminatingDriver = FALSE;
 	FailIf (NULL == controlInterface, Exit);
@@ -575,6 +594,7 @@ IOReturn AppleUSBAudioDevice::createControlsForInterface (IOAudioEngine *audioEn
 		FailWithAction (0 == featureUnitID, result = kIOReturnSuccess, Exit);	// There isn't a feature unit connected to this output terminal
 		// The interface was opened in AppleUSBAudioEngine::initHardware
 		controlInterface->SetAlternateInterface (this, kRootAlternateSetting);
+		debug2IOLog ("There are %d controls\n", usbAudio->GetNumChannels (interfaceNum, altInterfaceNum));
 		for (channelNum = 0; channelNum <= usbAudio->GetNumChannels (interfaceNum, altInterfaceNum); channelNum++) {
 			if (usbAudio->ChannelHasVolumeControl (controlInterfaceNum, 0, featureUnitID, channelNum)) {
 				micControl = AppleUSBAudioLevelControl::create (featureUnitID,
@@ -625,11 +645,7 @@ IOReturn AppleUSBAudioDevice::activateAudioEngine (IOAudioEngine *audioEngine, b
 
     debug5IOLog ("+AppleUSBAudioDevice[%p]::activateAudioEngine (%p, %d) - rc=%d\n", this, audioEngine, shouldStartAudioEngine, getRetainCount());
 
-    FailIf (kIOReturnSuccess != super::activateAudioEngine (audioEngine, shouldStartAudioEngine), Exit);
-	result = TRUE;
-
-Exit:
-	flushAudioControls ();
+    result = super::activateAudioEngine (audioEngine, shouldStartAudioEngine);
 
     debug4IOLog ("-AppleUSBAudioDevice[%p]::activateAudioEngine (%p) - rc=%d\n", this, audioEngine, getRetainCount ());
     return result;
