@@ -11,14 +11,36 @@
 #ifndef _APPLEUSBAUDIOENGINE_H
 #define _APPLEUSBAUDIOENGINE_H
 
+#include <libkern/OSByteOrder.h>
+#include <libkern/c++/OSCollectionIterator.h>
+#include <libkern/c++/OSMetaClass.h>
+
+#include <IOKit/IOLib.h>
+#include <IOKit/IOSyncer.h>
+#include <IOKit/IOService.h>
+#include <IOKit/IOMessage.h>
+#include <IOKit/IOMemoryCursor.h>
+#include <IOKit/IOMemoryDescriptor.h>
 #include <IOKit/IOBufferMemoryDescriptor.h>
+#include <IOKit/IOTimerEventSource.h>
 #include <IOKit/IOMultiMemoryDescriptor.h>
+
+#include <IOKit/audio/IOAudioDevice.h>
+#include <IOKit/audio/IOAudioPort.h>
+#include <IOKit/audio/IOAudioTypes.h>
+#include <IOKit/audio/IOAudioDefines.h>
+#include <IOKit/audio/IOAudioLevelControl.h>
 #include <IOKit/audio/IOAudioEngine.h>
 #include <IOKit/audio/IOAudioStream.h>
+
 #include <IOKit/usb/IOUSBPipe.h>
 #include <IOKit/usb/IOUSBDevice.h>
 #include <IOKit/usb/IOUSBInterface.h>
-#include <IOKit/IOTimerEventSource.h>
+
+#include "AppleUSBAudioCommon.h"
+#include "AppleUSBAudioDevice.h"
+#include "AppleUSBAudioLevelControl.h"
+#include "AppleUSBAudioMuteControl.h"
 
 #include "USBAudioObject.h"
 #include "AppleiSubEngine.h"
@@ -46,6 +68,7 @@ class AppleUSBAudioDevice;
 
 class IOSyncer;
 class AppleUSBAudioEngine;
+class AppleUSBAudioPlugin;
 
 typedef struct FrameListWriteInfo {
     AppleUSBAudioEngine *				audioEngine;
@@ -70,13 +93,17 @@ public:
     virtual IOReturn performAudioEngineStart ();
     virtual IOReturn performAudioEngineStop ();
     
-    virtual IOAudioEngineState _setState (IOAudioEngineState newState);
+//    virtual IOAudioEngineState _setState (IOAudioEngineState newState);
+	static void requestiSubClose (IOAudioEngine * audioEngine);
 
 	static void sampleRateHandler (void * target, void * parameter, IOReturn result, IOUSBIsocFrame * pFrames);
 
     static void readHandler (void * object, void * frameListIndex, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
     static void writeHandler (void * object, void * parameter, IOReturn result, IOUSBLowLatencyIsocFrame * pFrames);
 	static IOReturn iSubAttachChangeHandler (IOService *target, IOAudioControl *passThruControl, SInt32 oldValue, SInt32 newValue);
+	virtual IOReturn pluginDeviceRequest (IOUSBDevRequest * request, IOUSBCompletion * completion);
+	virtual void registerPlugin (AppleUSBAudioPlugin * thePlugin);
+	static void	pluginLoaded (AppleUSBAudioEngine * usbAudioEngineObject);
 
 protected:
 	UInt64								usbFrameToQueueAt;
@@ -109,6 +136,7 @@ protected:
 	UInt32 *							aveSampleRateBuf;		// only needs to be 3 bytes for the 10.14 value
 	IOTimerEventSource *				startTimer;
 	thread_call_t						iSubTeardownThreadCall;
+	thread_call_t						mPluginInitThread;
 	thread_call_t						iSubOpenThreadCall;
 	thread_call_t						firstUSBFrameCompletionThread;
 	IOAudioSampleRate					curSampleRate;
@@ -118,7 +146,6 @@ protected:
 	UInt32								theStartBufferSize;
 	UInt32								previous;
 	UInt32								previousClippedToFrame;
-	UInt32								theMaxPacket;
 	UInt32								readUSBFrameListSize;
 	UInt32								bufferOffset;
 	UInt32								averageSampleRate;
@@ -150,6 +177,9 @@ protected:
 	Boolean								terminatingDriver;
 	IOAudioStream *						mainStream;
 
+	IONotifier *						mPluginNotification;
+	AppleUSBAudioPlugin *				mPlugin;
+
 	// For iSub
 	IOMemoryDescriptor *				iSubBufferMemory;
 	UInt32								iSubLoopCount;
