@@ -1,21 +1,4 @@
-#include <sys/cdefs.h>
-
-__BEGIN_DECLS
-#include <kern/thread_call.h>
-__END_DECLS
-
 #include "AppleUSBAudioLevelControl.h"
-#include "AppleUSBAudioCommon.h"
-#include "AppleUSBAudioDevice.h"
-
-#include <IOKit/audio/IOAudioTypes.h>
-#include <IOKit/audio/IOAudioDefines.h>
-#include <IOKit/usb/USB.h>
-#include <IOKit/usb/IOUSBInterface.h>
-#include <IOKit/IOLib.h>
-#include <IOKit/IOPlatformExpert.h>
-
-#include <libkern/OSByteOrder.h>
 
 #define super IOAudioLevelControl
 
@@ -132,7 +115,7 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 
 	if (kIOAudioLevelControlSubTypeLFEVolume == subType) {
 		currentValue = currentValue / 2;
-		updateUSBValue (currentValue);
+		//updateUSBValue (currentValue);
 	}
 
 	// Set values needed to compute proper PRAM boot beep volume setting
@@ -170,6 +153,10 @@ bool AppleUSBAudioLevelControl::init (UInt8 theUnitID, UInt8 theInterfaceNumber,
 
 	FailIf (FALSE == super::init (currentValue, deviceMin, deviceMax, deviceMinDB, deviceMaxDB, theChannelNumber, channelName, 0, subType, usage), Exit);
 
+	if (kIOAudioLevelControlSubTypeLFEVolume == subType) {
+		updateUSBValue (currentValue);
+	}
+
 	result = TRUE;
 
 Exit:
@@ -231,8 +218,9 @@ void AppleUSBAudioLevelControl::updateUSBValue (SInt32 newValue) {
 			newVolume = (newValue * volRes) - offset;
 		}
 	}
+	debug2IOLog ("volume value is 0x%X\n", newVolume);
     theValue = HostToUSBWord (newVolume);
-	debug2IOLog ("setting volume to 0x%X\n", newVolume);
+	debug2IOLog ("setting volume to 0x%X (little endian)\n", newVolume);
 	ret = SetCurVolume (interfaceNumber, channelNumber, theValue);
 
 	if (getSubType () == kIOAudioLevelControlSubTypeLFEVolume) {
