@@ -38,6 +38,7 @@ void AppleiSubEngine::closeiSub (IOService * forClient) {
 	
 			if (NULL != iSubVolumeControl) {
 				result = audioEngine->removeDefaultAudioControl (iSubVolumeControl);
+				iSubVolumeControl->release ();
 				iSubVolumeControl = NULL;
 				#if DEBUGLOG
 				if (kIOReturnSuccess != result) IOLog ("Error 0x%x removing left iSub control\n", result);
@@ -46,6 +47,8 @@ void AppleiSubEngine::closeiSub (IOService * forClient) {
 	
 			if (NULL != muteControl) {
 				result = audioEngine->removeDefaultAudioControl (muteControl);
+				//muteControl->release ();
+				muteControl->release ();
 				muteControl = NULL;
 				#if DEBUGLOG
 				if (kIOReturnSuccess != result) IOLog ("Error 0x%x removing mute iSub control\n", result);
@@ -103,7 +106,7 @@ void AppleiSubEngine::free (void) {
 }
 
 // This has to be called on the audio engine's work loop so that we don't interrupt the IOAudioFamily
-bool AppleiSubEngine::openiSub (IOService * forClient) {
+bool AppleiSubEngine::openiSub (IOService * forClient, iSubRequestCloseRoutine theiSubRequestCloseRoutine) {
     bool						resultCode;
 
 	resultCode = FALSE;
@@ -113,6 +116,9 @@ bool AppleiSubEngine::openiSub (IOService * forClient) {
 	audioEngine = OSDynamicCast (IOAudioEngine, forClient);
 	FailIf (NULL == audioEngine, Exit);
 
+	FailIf (NULL == theiSubRequestCloseRoutine, Exit);
+	miSubRequestCloseRoutine = theiSubRequestCloseRoutine;
+
 //	audioEngine->joinPMtree (this);
 //	if (pm_vars != NULL) {
 //		registerPowerDriver (this, iSubPowerStates, NUM_POWER_STATES);
@@ -294,12 +300,26 @@ Exit:
 	return resultBool;
 }
 
+void AppleiSubEngine::stop (IOService *provider) {
+	debugIOLog("+AppleiSubEngine::stop\n");
+
+	PMstop();
+
+	super::stop(provider);
+
+	debugIOLog("-AppleiSubEngine::stop\n");
+}
+
 bool AppleiSubEngine::willTerminate (IOService * provider, IOOptionBits options) {
 	bool					resultCode;
 
 	debug5IOLog ("+AppleiSubEngine[%p]::willTerminate (%p, 0x%x) rc =\n", this, provider, options, getRetainCount ());
 
-        terminatingDriver = TRUE;
+	terminatingDriver = TRUE;
+
+	if (NULL != audioEngine) {
+		(miSubRequestCloseRoutine)(audioEngine);
+	}
 
 	if (FALSE == iSubUSBRunning) {
 		// close our stream interface and go away
@@ -327,6 +347,8 @@ IOReturn AppleiSubEngine::setPowerState (unsigned long powerStateOrdinal, IOServ
 
 	debug4IOLog("AppleiSubEngine[%p]::setPowerState (%lu, %p)\n", this, powerStateOrdinal, device);
 
+	result = kIOReturnError;
+
 	if (device == this) {
 		switch (powerStateOrdinal) {
 			case 0:		// Power off state
