@@ -43,14 +43,14 @@ OSDefineMetaClassAndStructors (USBAudioConfigObject, OSObject);
 /*
  	Public methods
 */
-USBAudioConfigObject * USBAudioConfigObject::create (const IOUSBConfigurationDescriptor * newConfigurationDescriptor) {
+USBAudioConfigObject * USBAudioConfigObject::create (const IOUSBConfigurationDescriptor * newConfigurationDescriptor, UInt8 controlInterfaceNum) {
     USBAudioConfigObject *			configObject;
 
     configObject = new USBAudioConfigObject;
 
     FailIf (NULL == configObject, Exit);
 
-    if (FALSE == configObject->init (newConfigurationDescriptor)) {
+    if (FALSE == configObject->init (newConfigurationDescriptor, controlInterfaceNum)) {
         configObject->release();
         configObject = 0;
     }
@@ -78,13 +78,15 @@ void USBAudioConfigObject::DumpConfigMemoryToIOLog (void) {
 }
 #endif
 
-bool USBAudioConfigObject::init (const IOUSBConfigurationDescriptor * newConfigurationDescriptor) {
+bool USBAudioConfigObject::init (const IOUSBConfigurationDescriptor * newConfigurationDescriptor, UInt8 controlInterfaceNum) {
     Boolean				result;
 
     result = FALSE;
     FailIf (FALSE == OSObject::init (), Exit);
     FailIf (NULL == newConfigurationDescriptor, Exit);
 
+	theControlInterfaceNum = controlInterfaceNum;
+
     debug2IOLog ("allocating %d bytes for config descriptor\n", USBToHostWord (newConfigurationDescriptor->wTotalLength));
     theConfigurationDescriptorPtr = (IOUSBConfigurationDescriptor *)IOMalloc (USBToHostWord (newConfigurationDescriptor->wTotalLength) + 1);
     FailIf (NULL == theConfigurationDescriptorPtr, Exit);
@@ -96,9 +98,13 @@ bool USBAudioConfigObject::init (const IOUSBConfigurationDescriptor * newConfigu
 #endif
 
     ParseConfigurationDescriptor ();
-    result = TRUE;
+	result = TRUE;
 
 Exit:
+	if (NULL != theConfigurationDescriptorPtr) {
+		IOFree (theConfigurationDescriptorPtr, USBToHostWord (newConfigurationDescriptor->wTotalLength) + 1);
+		theConfigurationDescriptorPtr = NULL;
+	}
     return result;
 }
 
@@ -113,7 +119,10 @@ void USBAudioConfigObject::free (void) {
 		theStreams = NULL;
 	}
 
-    IOFree (theConfigurationDescriptorPtr, USBToHostWord (theConfigurationDescriptorPtr->wTotalLength) + 1);
+	if (NULL != theConfigurationDescriptorPtr) {
+		IOFree (theConfigurationDescriptorPtr, USBToHostWord (theConfigurationDescriptorPtr->wTotalLength) + 1);
+	}
+
     OSObject::free ();
 }
 
@@ -270,6 +279,25 @@ UInt8 USBAudioConfigObject::GetFirstStreamInterfaceNum (void) {
     return interfaceNum;
 }
 
+void USBAudioConfigObject::GetControlledStreamNumbers (UInt8 **controledStreams, UInt8 *numControledStreams) {
+    USBAudioControlObject * 			thisControl;
+
+	*controledStreams = 0;
+	*numControledStreams = 0;
+	thisControl = NULL;
+
+	if (NULL != theControls) {
+		thisControl = OSDynamicCast (USBAudioControlObject, theControls->getObject (0));
+	} else {
+		debugIOLog ("couldn't get the control object\n");
+	}
+
+	if (thisControl) {
+		*controledStreams = thisControl->GetStreamInterfaceNumbers ();
+		*numControledStreams = thisControl->GetNumStreamInterfaces ();
+	}
+}
+
 UInt8 USBAudioConfigObject::GetControlInterfaceNum (void) {
     USBAudioControlObject * 			thisControl;
 	UInt8								interfaceNum;
@@ -940,8 +968,16 @@ void USBAudioConfigObject::ParseConfigurationDescriptor (void) {
     USBInterfaceDescriptorPtr			theInterfacePtr;
     USBAudioControlObject *				theControlObject;
     USBAudioStreamObject *				theStreamObject;
+	UInt8 *								streamInterfaceNumbers;
+	UInt8								thisInterfaceNumber;
+	UInt8								lastInterfaceNumber;
+	UInt8								numStreamInterfaces;
     UInt8								interfaceClass;
     UInt8								interfaceSubClass;
+	UInt8								numParsedInterfaces;
+	UInt8								index;
+	bool								haveControlInterface;
+	bool								foundStreamInterface;
 
     FailIf (NULL == theConfigurationDescriptorPtr, Exit);
     FailIf (0 == theConfigurationDescriptorPtr->bLength, Exit);
@@ -950,40 +986,75 @@ void USBAudioConfigObject::ParseConfigurationDescriptor (void) {
 	theControlObject = NULL;
 	theStreamObject = NULL;
     theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theConfigurationDescriptorPtr + theConfigurationDescriptorPtr->bLength);
-    while (0 != theInterfacePtr->bLength) {
-        switch (theInterfacePtr->bDescriptorType) {
-            case INTERFACE:
-                debugIOLog ("in INTERFACE in ParseConfigurationDescriptor\n");
-                theInterfacePtr = ParseInterfaceDescriptor (theInterfacePtr, &interfaceClass, &interfaceSubClass);
-                debug3IOLog ("interfaceClass = %d, interfaceSubClass = %d\n", interfaceClass, interfaceSubClass);
-                break;
-            case CS_INTERFACE:
-                debugIOLog ("in CS_INTERFACE in ParseConfigurationDescriptor\n");
-				if (AUDIOCONTROL == interfaceSubClass) {
-					debugIOLog ("found a AUDIOCONTROL CS_INTERFACE \n");
-					if (NULL != theControls) {
-						theControlObject = OSDynamicCast (USBAudioControlObject, theControls->getLastObject ());
-					}
-					FailIf (NULL == theControlObject, Exit);
-					theInterfacePtr = theControlObject->ParseACInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
-				} else if (AUDIOSTREAMING == interfaceSubClass) {
-					debugIOLog ("found a AUDIOSTREAMING CS_INTERFACE \n");
-					if (NULL != theStreams) {
-						theStreamObject = OSDynamicCast (USBAudioStreamObject, theStreams->getLastObject ());
-					}
-					FailIf (NULL == theStreamObject, Exit);
-					theInterfacePtr = theStreamObject->ParseASInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
-				} else {
-					// could be MIDI! -- dsw
-					theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+	numParsedInterfaces = 0;
+	numStreamInterfaces = 0;
+	lastInterfaceNumber = 0;
+	haveControlInterface = FALSE;
+	foundStreamInterface = FALSE;
+    while (theInterfacePtr && 0 != theInterfacePtr->bLength) {
+        if (INTERFACE == ((ACInterfaceDescriptorPtr)theInterfacePtr)->bDescriptorType) {
+			debugIOLog ("in INTERFACE in ParseConfigurationDescriptor\n");
+			thisInterfaceNumber = ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber;
+			theInterfacePtr = ParseInterfaceDescriptor (theInterfacePtr, &interfaceClass, &interfaceSubClass);
+
+			debug3IOLog ("theControlInterfaceNum = %d, thisInterfaceNumber = %d\n", theControlInterfaceNum, thisInterfaceNumber);
+			if (AUDIOCONTROL == interfaceSubClass && theControlInterfaceNum == thisInterfaceNumber) {
+				debugIOLog ("found a AUDIOCONTROL CS_INTERFACE \n");
+				if (NULL != theControls) {
+					theControlObject = OSDynamicCast (USBAudioControlObject, theControls->getLastObject ());
 				}
-                break;
-            default:
-                debugIOLog ("in default in ParseConfigurationDescriptor\n");
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+				FailIf (NULL == theControlObject, Exit);
+				theInterfacePtr = theControlObject->ParseACInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
+				GetControlledStreamNumbers (&streamInterfaceNumbers, &numStreamInterfaces);
+				haveControlInterface = TRUE;
+			} else if (haveControlInterface && AUDIOSTREAMING == interfaceSubClass) {
+				debugIOLog ("in CS_INTERFACE in ParseConfigurationDescriptor\n");
+				for (index = 0; index < numStreamInterfaces; index++) {
+					debug3IOLog ("comparing thisInterfaceNum = %d with %d\n", thisInterfaceNumber, streamInterfaceNumbers[index]);
+					if (thisInterfaceNumber == streamInterfaceNumbers[index]) {
+						debugIOLog ("found a AUDIOSTREAMING CS_INTERFACE \n");
+						if (NULL != theStreams) {
+							theStreamObject = OSDynamicCast (USBAudioStreamObject, theStreams->getLastObject ());
+						}
+						FailIf (NULL == theStreamObject, Exit);
+						theInterfacePtr = theStreamObject->ParseASInterfaceDescriptor (theInterfacePtr, ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber);
+						foundStreamInterface = TRUE;
+						break;			// Get out of for loop
+					}
+				}
+				if (thisInterfaceNumber != lastInterfaceNumber) {
+					lastInterfaceNumber = thisInterfaceNumber;
+					numParsedInterfaces++;
+					if (numParsedInterfaces > numStreamInterfaces) {
+						break;		// Get out of while loop, we've parsed everything associated with this control interface
+					}
+				}
+			} else if (MIDISTREAMING == interfaceSubClass) {
+				debug2IOLog ("MIDI, jumping forward %d bytes\n", theInterfacePtr->bLength);
+				for (index = 0; index < numStreamInterfaces; index++) {
+					if (thisInterfaceNumber == streamInterfaceNumbers[index]) {
+						break;
+					}
+				}
+				theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+			} else if (AUDIOCONTROL == interfaceSubClass) {
+				debugIOLog ("Found a control interface that we don't care about\n");
+				debug2IOLog ("jumping forward %d bytes\n", ((((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[1] << 8) | (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[0])));
+				theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + ((((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[1] << 8) | (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->wTotalLength[0])));
+			} else {
+				debug2IOLog ("Unknown, jumping forward %d bytes\n", theInterfacePtr->bLength);
+				theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+			}
+		} else {
+			debug2IOLog ("in default in ParseConfigurationDescriptor, jumping forward %d bytes\n", theInterfacePtr->bLength);
+			theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
         }
     }
 
+	if (theControlObject && FALSE == foundStreamInterface) {
+		theControls->removeObject (theControls->getCount () - 1);
+	}
+
 Exit:
     return;
 }
@@ -996,7 +1067,6 @@ USBInterfaceDescriptorPtr USBAudioConfigObject::ParseInterfaceDescriptor (USBInt
 
 	FailIf (NULL == theInterfacePtr, Exit);
 	FailIf (0 == theInterfacePtr->bLength, Exit);
-	FailIf (INTERFACE != theInterfacePtr->bDescriptorType, Exit);
 
 	if (NULL != interfaceClass)
 		*interfaceClass = theInterfacePtr->bInterfaceClass;
@@ -1005,25 +1075,26 @@ USBInterfaceDescriptorPtr USBAudioConfigObject::ParseInterfaceDescriptor (USBInt
 
 	if (AUDIOCONTROL == theInterfacePtr->bInterfaceSubClass) {
 		debugIOLog ("found a AUDIOCONTROL interface\n");
-		theControlObject = USBAudioControlObject::create ();
+		if (theControlInterfaceNum == theInterfacePtr->bDescriptorSubtype) {
+			theControlObject = USBAudioControlObject::create ();
 
-		FailIf (NULL == theControlObject, Exit);
-		theControlObject->SetInterfaceNumber (theInterfacePtr->bDescriptorSubtype);
-		theControlObject->SetAlternateSetting (theInterfacePtr->bAlternateSetting);
-		theControlObject->SetNumEndpoints (theInterfacePtr->bNumEndpoints);
-		theControlObject->SetInterfaceClass (theInterfacePtr->bInterfaceClass);
-		theControlObject->SetInterfaceSubClass (theInterfacePtr->bInterfaceSubClass);
-		theControlObject->SetInterfaceProtocol (theInterfacePtr->bInterfaceProtocol);
-		theControlInterfaceNum = ((ACInterfaceDescriptorPtr)theInterfacePtr)->bInterfaceNumber;
+			FailIf (NULL == theControlObject, Exit);
+			theControlObject->SetInterfaceNumber (theInterfacePtr->bDescriptorSubtype);
+			theControlObject->SetAlternateSetting (theInterfacePtr->bAlternateSetting);
+			theControlObject->SetNumEndpoints (theInterfacePtr->bNumEndpoints);
+			theControlObject->SetInterfaceClass (theInterfacePtr->bInterfaceClass);
+			theControlObject->SetInterfaceSubClass (theInterfacePtr->bInterfaceSubClass);
+			theControlObject->SetInterfaceProtocol (theInterfacePtr->bInterfaceProtocol);
 
-		if (NULL == theControls) {
-			theControls = OSArray::withObjects ((const OSObject **)&theControlObject, 1);
-			FailIf (NULL == theControls, Exit);
-		} else {
-			theControls->setObject (theControlObject);
+			if (NULL == theControls) {
+				theControls = OSArray::withObjects ((const OSObject **)&theControlObject, 1);
+				FailIf (NULL == theControls, Exit);
+			} else {
+				theControls->setObject (theControlObject);
+			}
+
+			theControlObject->release ();
 		}
-
-		theControlObject->release ();
 	} else if (AUDIOSTREAMING == theInterfacePtr->bInterfaceSubClass) {
 		debugIOLog ("found a AUDIOSTREAMING interface\n");
 		theStreamObject = USBAudioStreamObject::create ();
@@ -1522,6 +1593,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 	USBFeatureUnitObject *		featureUnit;
 	USBMixerUnitObject *		mixerUnit;
 	USBSelectorUnitObject *		selectorUnit;
+	UInt8						index;
 
 	debug3IOLog ("+USBAudioControlObject::ParseACInterfaceDescriptor (%p, %d)\n", theInterfacePtr, currentInterface);
 
@@ -1533,10 +1605,19 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
         switch (theInterfacePtr->bDescriptorSubtype) {
             case HEADER:
                 debugIOLog ("in HEADER in ParseACInterfaceDescriptor\n");
-                adcVersion = USBToHostWord (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->bcdADC);
+                adcVersion = USBToHostWord (((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->bcdADC[0]);
+				if (adcVersion != 0x0100) {
+				}
+				// !!!Need to check the adcVersion to make sure that it's 0x0100 or else we don't know how to parse this descriptor!!!
                 numStreamInterfaces = ((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->bInCollection;
-                firstASInterface = ((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->baInterfaceNr[1];
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
+				debug2IOLog ("numStreamInterfaces = %d\n", numStreamInterfaces);
+				streamInterfaceNumbers = (UInt8 *)IOMalloc (numStreamInterfaces);
+				memcpy (streamInterfaceNumbers, ((ACInterfaceHeaderDescriptorPtr)theInterfacePtr)->baInterfaceNr, numStreamInterfaces);
+				debugIOLog ("they are: ");
+				for (index = 0; index < numStreamInterfaces; index++) {
+					debug2IOLog ("%d, ", streamInterfaceNumbers[index]);
+				}
+				debugIOLog ("\n");
                 break;
             case INPUT_TERMINAL:
                 debugIOLog ("in INPUT_TERMINAL in ParseACInterfaceDescriptor\n");
@@ -1556,7 +1637,6 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				}
 
 				inputTerminal->release ();
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
                 break;
             case OUTPUT_TERMINAL:
                 debugIOLog ("in OUTPUT_TERMINAL in ParseACInterfaceDescriptor\n");
@@ -1575,7 +1655,6 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				}
 
 				outputTerminal->release ();
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
                 break;
             case FEATURE_UNIT:
 				{
@@ -1589,6 +1668,7 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
                 featureUnit->SetControlSize (((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bControlSize);
                 // subtract 7 because that's how many fields are guaranteed to be in the struct
 				numControls = (((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bLength - 7) / ((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bControlSize;
+				debug2IOLog ("There are %d controls on this feature unit\n", numControls);
 				featureUnit->InitControlsArray (&((ACFeatureUnitDescriptorPtr)theInterfacePtr)->bmaControls[0], numControls);
 
 				if (NULL == theFeatureUnits) {
@@ -1599,7 +1679,6 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				}
 
 				featureUnit->release ();
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
                 break;
 				}
             case MIXER_UNIT:
@@ -1635,7 +1714,6 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				}
 
 				mixerUnit->release ();
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
 				break;
 				}
             case SELECTOR_UNIT:
@@ -1654,20 +1732,17 @@ USBInterfaceDescriptorPtr USBAudioControlObject::ParseACInterfaceDescriptor (USB
 				}
 
 				selectorUnit->release ();
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
 				break;
             case PROCESSING_UNIT:
                 debugIOLog ("in PROCESSING_UNIT in ParseACInterfaceDescriptor\n");
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
 				break;
             case EXTENSION_UNIT:
                 debugIOLog ("in EXTENSION_UNIT in ParseACInterfaceDescriptor\n");
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
 				break;
             default:
                 debugIOLog ("in default in ParseACInterfaceDescriptor\n");
-                theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
         }
+		theInterfacePtr = (USBInterfaceDescriptorPtr)((UInt8 *)theInterfacePtr + theInterfacePtr->bLength);
     }
 
 Exit:
@@ -1815,7 +1890,7 @@ USBInterfaceDescriptorPtr USBAudioStreamObject::ParseASInterfaceDescriptor (USBI
 
     FailIf (NULL == theInterfacePtr, Exit);
     FailIf (0 == theInterfacePtr->bLength, Exit);
-    FailIf (CS_INTERFACE != theInterfacePtr->bDescriptorType, Exit);
+//    FailIf (CS_INTERFACE != theInterfacePtr->bDescriptorType, Exit);
 
     done = FALSE;
     while (theInterfacePtr->bLength && !done) {
