@@ -272,8 +272,8 @@ typedef struct ACInterfaceHeaderDescriptor {
     UInt8									bLength;
     UInt8									bDescriptorType;
     UInt8									bDescriptorSubtype;
-    UInt8									bcdADC;
-    UInt16									wTotalLength; 
+    UInt8									bcdADC[2];					// Not PPC aligned
+    UInt8									wTotalLength[2];			// Not PPC aligned
     UInt8									bInCollection;
     UInt8									baInterfaceNr[1];			// There are bInCollection of these
 } ACInterfaceHeaderDescriptor, *ACInterfaceHeaderDescriptorPtr;
@@ -586,7 +586,7 @@ private:
     UInt8							interfaceSubClass;	// should always be 1
     UInt8							interfaceProtocol;
     UInt8							numStreamInterfaces;
-    UInt8							firstASInterface;
+	UInt8 *							streamInterfaceNumbers;
 
 	USBFeatureUnitObject *			GetFeatureUnitObject (UInt8 unitID);
 	USBFeatureUnitObject *			GetIndexedFeatureUnitObject (UInt8 index);
@@ -617,7 +617,7 @@ public:
 	UInt8							GetFeatureSourceID (UInt8 featureUnitID);
 	UInt8							GetFeatureUnitID (UInt8 featureUnitID);
 	UInt8							GetFeatureUnitIDConnectedToOutputTerminal (UInt8 outputTerminalID);
-	UInt8							GetFirstASInterface (void) {return firstASInterface;}
+//	UInt8							GetFirstASInterface (void) {return firstASInterface;}
 	UInt8							GetIndexedFeatureUnitID (UInt8 featureUnitIndex);
 	UInt8							GetIndexedMixerUnitID (UInt8 mixerUnitIndex);
 	UInt8							GetIndexedSelectorUnitID (UInt8 selectorUnitIndex);
@@ -635,6 +635,7 @@ public:
 	UInt8							GetNumOutputTerminals (void);
 	UInt8							GetNumSelectorSources (UInt8 unitID);
 	UInt8							GetNumStreamInterfaces (void) {return numStreamInterfaces;}
+	UInt8 *							GetStreamInterfaceNumbers (void) {return streamInterfaceNumbers;}
 	UInt16							GetOutputTerminalType (UInt8 terminalID);
 	UInt8 *							GetSelectorSources (UInt8 unitID);
 	Boolean							MasterHasMuteControl (UInt8 featureUnitID);
@@ -785,8 +786,8 @@ private:
     OSArray *						theStreams;
 
 public:
-    static USBAudioConfigObject *	 create (const IOUSBConfigurationDescriptor * newConfigurationDescriptor);
-    virtual bool					init (const IOUSBConfigurationDescriptor * newConfigurationDescriptor);
+    static USBAudioConfigObject *	create (const IOUSBConfigurationDescriptor * newConfigurationDescriptor, UInt8 controlInterfaceNum);
+    virtual bool					init (const IOUSBConfigurationDescriptor * newConfigurationDescriptor, UInt8 controlInterfaceNum);
     virtual void					free (void);
 
 	Boolean							ChannelHasMuteControl (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 featureUnitID, UInt8 channelNum);
@@ -798,6 +799,7 @@ public:
 	UInt32							GetAC3BSID (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt8							GetFeatureUnitIDConnectedToOutputTerminal (UInt8 interfaceNum, UInt8 altInterfaceNum, UInt8 outputTerminalID);
     UInt8							GetFirstStreamInterfaceNum (void);
+	void							GetControlledStreamNumbers (UInt8 **controledStreams, UInt8 *numControledStreams);
     UInt8							GetControlInterfaceNum (void);
 	UInt16							GetFormat (UInt8 interfaceNum, UInt8 altInterfaceNum);
 	UInt32							GetHighestSampleRate (UInt8 interfaceNum, UInt8 altInterfaceNum);
